@@ -1379,6 +1379,13 @@ brasero_burn_dialog_finalize (GObject * object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+/**
+ * brasero_burn_dialog_new:
+ *
+ * Creates a new #BraseroBurnDialog object
+ *
+ * Return value: a #GtkWidget. Unref when it is not needed anymore.
+ **/
 GtkWidget *
 brasero_burn_dialog_new (void)
 {
@@ -1893,7 +1900,11 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog)
 	brasero_burn_dialog_activity_stop (dialog, primary);
 	g_free (primary);
 
-	if (!brasero_burn_session_is_dest_file (priv->session)) {
+	/* Don't show the "Make Another Copy" button if:
+	 * - we wrote to a file
+	 * - we wrote a merged session */
+	if (!brasero_burn_session_is_dest_file (priv->session)
+	&&!(brasero_burn_session_get_flags (priv->session) & BRASERO_BURN_FLAG_MERGE)) {
 		/* Useful button but it shouldn't be used for images */
 		make_another = gtk_dialog_add_button (GTK_DIALOG (dialog),
 						      _("Make _Another Copy"),
@@ -2220,6 +2231,15 @@ brasero_burn_dialog_wait_for_ready_state (BraseroBurnDialog *dialog)
 	return (result == BRASERO_BURN_OK);
 }
 
+/**
+ * brasero_burn_dialog_run:
+ * @dialog: a #BraseroBurnDialog
+ * @session: a #BraseroBurnSession
+ *
+ * Start burning the contents of @session.
+ *
+ * Return value: a #gboolean. TRUE if the operation was successfully carried out, FALSE otherwise.
+ **/
 gboolean
 brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 			 BraseroBurnSession *session)
@@ -2302,6 +2322,16 @@ brasero_burn_dialog_cancel_dialog (GtkWidget *toplevel)
 	return (result != GTK_RESPONSE_OK);
 }
 
+/**
+ * brasero_burn_dialog_cancel:
+ * @dialog: a #BraseroBurnDialog
+ * @force_cancellation: a #gboolean
+ *
+ * Cancel the ongoing operation run by @dialog; if @force_cancellation is FALSE then it can
+ * happen that the operation won't be cancelled if there is a risk to make a disc unusable.
+ *
+ * Return value: a #gboolean. TRUE if it was sucessfully cancelled, FALSE otherwise.
+ **/
 gboolean
 brasero_burn_dialog_cancel (BraseroBurnDialog *dialog,
 			    gboolean force_cancellation)
