@@ -945,13 +945,46 @@ brasero_burn_session_get_required_media_type (BraseroBurnSession *session)
 	return required_media;
 }
 
+guint
+brasero_burn_session_get_possible_output_formats (BraseroBurnSession *session,
+						  BraseroImageFormat *formats)
+{
+	guint num = 0;
+	BraseroImageFormat format;
+	BraseroTrackType *output = NULL;
+
+	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (session), 0);
+
+	/* see how many output format are available */
+	format = BRASERO_IMAGE_FORMAT_CDRDAO;
+	(*formats) = BRASERO_IMAGE_FORMAT_NONE;
+
+	output = brasero_track_type_new ();
+	brasero_track_type_set_has_image (output);
+
+	for (; format > BRASERO_IMAGE_FORMAT_NONE; format >>= 1) {
+		BraseroBurnResult result;
+
+		brasero_track_type_set_image_format (output, format);
+		result = brasero_burn_session_output_supported (session, output);
+		if (result == BRASERO_BURN_OK) {
+			(*formats) |= format;
+			num ++;
+		}
+	}
+
+	brasero_track_type_free (output);
+
+	return num;
+}
+
 BraseroImageFormat
 brasero_burn_session_get_default_output_format (BraseroBurnSession *session)
 {
 	BraseroBurnCaps *self;
-	BraseroTrackType source;
-	BraseroTrackType output;
 	BraseroBurnResult result;
+	BraseroTrackType source = { BRASERO_TRACK_TYPE_NONE, { 0, }};
+	BraseroTrackType output = { BRASERO_TRACK_TYPE_NONE, { 0, }};
 
 	self = brasero_burn_caps_get_default ();
 
@@ -1482,8 +1515,7 @@ brasero_burn_session_get_burn_flags (BraseroBurnSession *session,
 	BraseroBurnFlag session_flags;
 	/* FIXME: what's the meaning of NOGRACE when outputting ? */
 	BraseroBurnFlag compulsory_flags = BRASERO_BURN_FLAG_NONE;
-	BraseroBurnFlag supported_flags = BRASERO_BURN_FLAG_DONT_OVERWRITE|
-					  BRASERO_BURN_FLAG_CHECK_SIZE|
+	BraseroBurnFlag supported_flags = BRASERO_BURN_FLAG_CHECK_SIZE|
 					  BRASERO_BURN_FLAG_NOGRACE;
 
 	self = brasero_burn_caps_get_default ();
