@@ -63,7 +63,6 @@ struct _BraseroDataVFSPrivate
 
 	BraseroFilteredUri *filtered;
 
-	BraseroIO *io;
 	BraseroIOJobBase *load_uri;
 	BraseroIOJobBase *load_contents;
 
@@ -116,6 +115,41 @@ brasero_data_vfs_is_loading_uri (BraseroDataVFS *self)
 	return (g_hash_table_size (priv->loading) != 0);
 }
 
+static BraseroBurnResult
+brasero_data_vfs_emit_image_signal (BraseroDataVFS *self,
+				    const gchar *uri)
+{
+	GValue instance_and_params [2];
+	GValue return_value;
+	GValue *params;
+
+	/* object which signalled */
+	instance_and_params->g_type = 0;
+	g_value_init (instance_and_params, G_TYPE_FROM_INSTANCE (self));
+	g_value_set_instance (instance_and_params, self);
+
+	/* arguments of signal (name) */
+	params = instance_and_params + 1;
+	params->g_type = 0;
+	g_value_init (params, G_TYPE_STRING);
+	g_value_set_string (params, uri);
+
+	/* default to CANCEL */
+	return_value.g_type = 0;
+	g_value_init (&return_value, G_TYPE_INT);
+	g_value_set_int (&return_value, BRASERO_BURN_CANCEL);
+
+	g_signal_emitv (instance_and_params,
+			brasero_data_vfs_signals [IMAGE_SIGNAL],
+			0,
+			&return_value);
+
+	g_value_unset (instance_and_params);
+	g_value_unset (params);
+
+	return g_value_get_int (&return_value);
+}
+
 static gboolean
 brasero_data_vfs_check_uri_result (BraseroDataVFS *self,
 				   const gchar *uri,
@@ -240,7 +274,6 @@ brasero_data_vfs_directory_load_end (GObject *object,
 		if (!parent)
 			continue;
 
-		parent->is_exploring = FALSE;
 		brasero_data_project_directory_node_loaded (BRASERO_DATA_PROJECT (self), parent);
 	}
 
@@ -413,6 +446,15 @@ brasero_data_vfs_directory_load_result (GObject *owner,
 		if (!parent)
 			continue;
 
+		if (parent->is_root) {
+			/* This may be true in some rare situations (when the root of a
+			 * volume has been added like burn:/// */
+			brasero_data_project_add_loading_node (BRASERO_DATA_PROJECT (self),
+							       uri,
+							       parent);
+			 continue;
+		}
+
 		if (g_file_info_get_is_symlink (info)) {
 			if (brasero_data_vfs_directory_check_symlink_loop (self, parent, uri, info)) {
 				brasero_data_project_exclude_uri (BRASERO_DATA_PROJECT (self), uri);
@@ -476,8 +518,7 @@ brasero_data_vfs_load_directory (BraseroDataVFS *self,
 							   NULL);
 
 	/* no need to require mime types here as these rows won't be visible */
-	brasero_io_load_directory (priv->io,
-				   uri,
+	brasero_io_load_directory (uri,
 				   priv->load_contents,
 				   BRASERO_IO_INFO_PERM|
 				  (priv->replace_sym ? BRASERO_IO_INFO_FOLLOW_SYMLINK:BRASERO_IO_INFO_NONE),
@@ -523,41 +564,6 @@ brasero_data_vfs_loading_node_end (GObject *object,
 			       g_hash_table_size (priv->directories));
 }
 
-static BraseroBurnResult
-brasero_data_vfs_emit_image_signal (BraseroDataVFS *self,
-				    const gchar *uri)
-{
-	GValue instance_and_params [2];
-	GValue return_value;
-	GValue *params;
-
-	/* object which signalled */
-	instance_and_params->g_type = 0;
-	g_value_init (instance_and_params, G_TYPE_FROM_INSTANCE (self));
-	g_value_set_instance (instance_and_params, self);
-
-	/* arguments of signal (name) */
-	params = instance_and_params + 1;
-	params->g_type = 0;
-	g_value_init (params, G_TYPE_STRING);
-	g_value_set_string (params, uri);
-
-	/* default to FALSE */
-	return_value.g_type = 0;
-	g_value_init (&return_value, G_TYPE_INT);
-	g_value_set_int (&return_value, BRASERO_BURN_CANCEL);
-
-	g_signal_emitv (instance_and_params,
-			brasero_data_vfs_signals [IMAGE_SIGNAL],
-			0,
-			&return_value);
-
-	g_value_unset (instance_and_params);
-	g_value_unset (params);
-
-	return g_value_get_int (&return_value);
-}
-
 static void
 brasero_data_vfs_loading_node_result (GObject *owner,
 				      GError *error,
@@ -601,6 +607,7 @@ brasero_data_vfs_loading_node_result (GObject *owner,
 	stats = BRASERO_FILE_NODE_STATS (root);
 
 	if (stats && !stats->children
+	&&  brasero_file_node_get_n_children (root) <= 1
 	&& (!strcmp (g_file_info_get_content_type (info), "application/x-toc")
 	||  !strcmp (g_file_info_get_content_type (info), "application/x-cdrdao-toc")
 	||  !strcmp (g_file_info_get_content_type (info), "application/x-cue")
@@ -709,8 +716,7 @@ brasero_data_vfs_load_node (BraseroDataVFS *self,
 						      brasero_data_vfs_loading_node_end,
 						      NULL);
 
-	brasero_io_get_file_info (priv->io,
-				  uri,
+	brasero_io_get_file_info (uri,
 				  priv->load_uri,
 				  flags|
 				  (priv->replace_sym ? BRASERO_IO_INFO_FOLLOW_SYMLINK:BRASERO_IO_INFO_NONE),
@@ -789,7 +795,6 @@ brasero_data_vfs_increase_priority_cb (gpointer data, gpointer user_data)
 static gboolean
 brasero_data_vfs_require_higher_priority (BraseroDataVFS *self,
 					  BraseroFileNode *node,
-					  BraseroIO *io,
 					  BraseroIOJobBase *type)
 {
 	gchar *registered;
@@ -799,8 +804,7 @@ brasero_data_vfs_require_higher_priority (BraseroDataVFS *self,
 	registered = brasero_utils_register_string (uri);
 	g_free (uri);
 
-	brasero_io_find_urgent (io,
-				type,
+	brasero_io_find_urgent (type,
 				brasero_data_vfs_increase_priority_cb,
 				registered);
 
@@ -817,7 +821,6 @@ brasero_data_vfs_require_directory_contents (BraseroDataVFS *self,
 	priv = BRASERO_DATA_VFS_PRIVATE (self);
 	return brasero_data_vfs_require_higher_priority (self,
 							 node,
-							 priv->io,
 							 priv->load_contents);
 }
 
@@ -830,7 +833,6 @@ brasero_data_vfs_require_node_load (BraseroDataVFS *self,
 	priv = BRASERO_DATA_VFS_PRIVATE (self);
 	return brasero_data_vfs_require_higher_priority (self,
 							 node,
-							 priv->io,
 							 priv->load_uri);
 }
 
@@ -871,8 +873,7 @@ brasero_data_vfs_load_mime (BraseroDataVFS *self,
 			ref_node = brasero_data_project_reference_get (BRASERO_DATA_PROJECT (self), reference);
 			if (ref_node == node) {
 				/* Ask for a higher priority */
-				brasero_io_find_urgent (priv->io,
-							priv->load_uri,
+				brasero_io_find_urgent (priv->load_uri,
 							brasero_data_vfs_increase_priority_cb,
 							registered);
 				brasero_utils_unregister_string (registered);
@@ -886,8 +887,7 @@ brasero_data_vfs_load_mime (BraseroDataVFS *self,
 		g_hash_table_insert (priv->loading, registered, nodes);
 
 		/* Yet, ask for a higher priority */
-		brasero_io_find_urgent (priv->io,
-					priv->load_uri,
+		brasero_io_find_urgent (priv->load_uri,
 					brasero_data_vfs_increase_priority_cb,
 					registered);
 		brasero_utils_unregister_string (registered);
@@ -977,13 +977,14 @@ brasero_data_vfs_clear (BraseroDataVFS *self)
 	priv = BRASERO_DATA_VFS_PRIVATE (self);
 
 	/* Stop all VFS operations */
-	if (priv->io) {
-		brasero_io_cancel_by_base (priv->io, priv->load_uri);
-		brasero_io_cancel_by_base (priv->io, priv->load_contents);
-
+	if (priv->load_uri) {
+		brasero_io_cancel_by_base (priv->load_uri);
 		g_free (priv->load_uri);
 		priv->load_uri = NULL;
+	}
 
+	if (priv->load_contents) {
+		brasero_io_cancel_by_base (priv->load_contents);
 		g_free (priv->load_contents);
 		priv->load_contents = NULL;
 	}
@@ -1123,9 +1124,6 @@ brasero_data_vfs_init (BraseroDataVFS *object)
 	/* create the hash tables */
 	priv->loading = g_hash_table_new (g_str_hash, g_str_equal);
 	priv->directories = g_hash_table_new (g_str_hash, g_str_equal);
-
-	/* get the vfs object */
-	priv->io = brasero_io_get_default ();
 }
 
 static void
@@ -1152,11 +1150,6 @@ brasero_data_vfs_finalize (GObject *object)
 		priv->filtered = NULL;
 	}
 
-	if (priv->io) {
-		g_object_unref (priv->io);
-		priv->io = NULL;
-	}
-
 	G_OBJECT_CLASS (brasero_data_vfs_parent_class)->finalize (object);
 }
 
