@@ -39,7 +39,6 @@
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
 
-#include <libhal.h>
 #include <gio/gio.h>
 
 #include "brasero-media-private.h"
@@ -48,37 +47,36 @@
 #include "brasero-medium.h"
 #include "brasero-volume.h"
 #include "brasero-drive.h"
-#include "burn-hal-watch.h"
 
+#include "scsi-device.h"
+#include "scsi-utils.h"
+#include "scsi-spc1.h"
 #include "scsi-mmc1.h"
-
-#if defined(HAVE_STRUCT_USCSI_CMD)
-#define DEVICE_MODEL	"info.product"
-#define BLOCK_DEVICE	"block.solaris.raw_device"
-#else
-#define DEVICE_MODEL	"storage.model"
-#define BLOCK_DEVICE	"block.device"
-#endif
+#include "scsi-mmc2.h"
+#include "scsi-status-page.h"
+#include "scsi-mode-pages.h"
+#include "scsi-sbc.h"
 
 typedef struct _BraseroDrivePrivate BraseroDrivePrivate;
 struct _BraseroDrivePrivate
 {
+	GDrive *gdrive;
+
+	GThread *probe;
+	gint probe_id;
+
 	BraseroMedium *medium;
 	BraseroDriveCaps caps;
-	gchar *name;
-	gchar *path;
-	gchar *block_path;
+
 	gchar *udi;
 
-	gint bus;
-	gint target;
-	gint lun;
-
-	gulong hal_sig;
+	gchar *path;
+	gchar *block_path;
 
 	GCancellable *cancel;
 
 	guint probed:1;
+	guint probe_cancelled:1;
 };
 
 #define BRASERO_DRIVE_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_DRIVE, BraseroDrivePrivate))
@@ -92,17 +90,13 @@ static gulong drive_signals [LAST_SIGNAL] = {0, };
 
 enum {
 	PROP_NONE	= 0,
-	PROP_DEVICE,
+	PROP_GDRIVE,
 	PROP_UDI
 };
 
 G_DEFINE_TYPE (BraseroDrive, brasero_drive, G_TYPE_OBJECT);
 
-/**
- * This is private API. The function is defined in brasero-volume.c
- */
-BraseroVolume *
-brasero_volume_new (BraseroDrive *drive, const gchar *udi);
+#define BRASERO_DRIVE_OPEN_ATTEMPTS			5
 
 /**
  * brasero_drive_get_gdrive:
@@ -115,11 +109,7 @@ brasero_volume_new (BraseroDrive *drive, const gchar *udi);
 GDrive *
 brasero_drive_get_gdrive (BraseroDrive *drive)
 {
-	const gchar *volume_path = NULL;
-	GVolumeMonitor *monitor;
-	GDrive *gdrive = NULL;
-	GList *drives;
-	GList *iter;
+	BraseroDrivePrivate *priv;
 
 	g_return_val_if_fail (drive != NULL, NULL);
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
@@ -127,43 +117,9 @@ brasero_drive_get_gdrive (BraseroDrive *drive)
 	if (brasero_drive_is_fake (drive))
 		return NULL;
 
-#if defined(HAVE_STRUCT_USCSI_CMD)
-	volume_path = brasero_drive_get_block_device (drive);
-#else
-	volume_path = brasero_drive_get_device (drive);
-#endif
+	priv = BRASERO_DRIVE_PRIVATE (drive);
 
-	/* NOTE: medium-monitor already holds a reference for GVolumeMonitor */
-	monitor = g_volume_monitor_get ();
-	drives = g_volume_monitor_get_connected_drives (monitor);
-	g_object_unref (monitor);
-
-	for (iter = drives; iter; iter = iter->next) {
-		gchar *device_path;
-		GDrive *tmp;
-
-		tmp = iter->data;
-		device_path = g_drive_get_identifier (tmp, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
-		if (!device_path)
-			continue;
-
-		BRASERO_MEDIA_LOG ("Found drive %s", device_path);
-		if (!strcmp (device_path, volume_path)) {
-			gdrive = tmp;
-			g_free (device_path);
-			g_object_ref (gdrive);
-			break;
-		}
-
-		g_free (device_path);
-	}
-	g_list_foreach (drives, (GFunc) g_object_unref, NULL);
-	g_list_free (drives);
-
-	if (!drive)
-		BRASERO_MEDIA_LOG ("No drive found for medium");
-
-	return gdrive;
+	return g_object_ref (priv->gdrive);
 }
 
 /**
@@ -178,7 +134,6 @@ brasero_drive_get_gdrive (BraseroDrive *drive)
 gboolean
 brasero_drive_can_eject (BraseroDrive *drive)
 {
-	GDrive *gdrive;
 	GVolume *volume;
 	gboolean result;
 	BraseroDrivePrivate *priv;
@@ -188,25 +143,20 @@ brasero_drive_can_eject (BraseroDrive *drive)
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
 
-	gdrive = brasero_drive_get_gdrive (drive);
-	if (!gdrive) {
+	if (!priv->gdrive) {
 		BRASERO_MEDIA_LOG ("No GDrive");
 		goto last_resort;
 	}
 
-	if (!g_drive_can_eject (gdrive)) {
+	if (!g_drive_can_eject (priv->gdrive)) {
 		BRASERO_MEDIA_LOG ("GDrive can't eject");
 		goto last_resort;
 	}
 
-	g_object_unref (gdrive);
 	return TRUE;
 
 last_resort:
 
-	if (gdrive)
-		g_object_unref (gdrive);
-
 	if (!priv->medium)
 		return FALSE;
 
@@ -239,7 +189,6 @@ brasero_drive_eject (BraseroDrive *drive,
 {
 	BraseroDrivePrivate *priv;
 	GVolume *gvolume;
-	GDrive *gdrive;
 	gboolean res;
 
 	g_return_val_if_fail (drive != NULL, FALSE);
@@ -247,13 +196,10 @@ brasero_drive_eject (BraseroDrive *drive,
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
 
-	gdrive = brasero_drive_get_gdrive (drive);
-	res = brasero_gio_operation_eject_drive (gdrive,
+	res = brasero_gio_operation_eject_drive (priv->gdrive,
 						 priv->cancel,
 						 wait,
 						 error);
-	g_object_unref (gdrive);
-
 	if (res)
 		return TRUE;
 
@@ -270,6 +216,25 @@ brasero_drive_eject (BraseroDrive *drive,
 	return res;
 }
 
+/**
+ * brasero_drive_cancel_current_operation:
+ * @drive: #BraseroDrive *
+ *
+ * Cancels all operations currently running for @drive
+ *
+ **/
+void
+brasero_drive_cancel_current_operation (BraseroDrive *drive)
+{
+	BraseroDrivePrivate *priv;
+
+	g_return_if_fail (drive != NULL);
+	g_return_if_fail (BRASERO_IS_DRIVE (drive));
+
+	priv = BRASERO_DRIVE_PRIVATE (drive);	
+	g_cancellable_cancel (priv->cancel);
+}
+
 /**
  * brasero_drive_get_bus_target_lun_string:
  * @drive: a #BraseroDrive
@@ -278,23 +243,16 @@ brasero_drive_eject (BraseroDrive *drive,
  * sometimes needed by some backends like cdrecord.
  *
  * Return value: a string or NULL. The string must be freed when not needed
+ *
+ * Deprecated since 2.27.3
  **/
 gchar *
 brasero_drive_get_bus_target_lun_string (BraseroDrive *drive)
 {
-	BraseroDrivePrivate *priv;
-
 	g_return_val_if_fail (drive != NULL, NULL);
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
-	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->udi)
-		return NULL;
-
-	if (priv->bus < 0)
-		return NULL;
-
-	return g_strdup_printf ("%i,%i,%i", priv->bus, priv->target, priv->lun);
+	return NULL;
 }
 
 /**
@@ -315,7 +273,7 @@ brasero_drive_is_fake (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	return (priv->path == NULL);
+	return (priv->gdrive == NULL);
 }
 
 /**
@@ -337,7 +295,7 @@ brasero_drive_is_door_open (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->udi)
+	if (!priv->gdrive)
 		return FALSE;
 
 	handle = brasero_device_handle_open (priv->path, FALSE, NULL);
@@ -396,46 +354,31 @@ brasero_drive_lock (BraseroDrive *drive,
 		    const gchar *reason,
 		    gchar **reason_for_failure)
 {
+	BraseroDeviceHandle *handle;
 	BraseroDrivePrivate *priv;
-	BraseroHALWatch *watch;
-	LibHalContext *ctx;
-	DBusError error;
+	const gchar *device;
 	gboolean result;
-	gchar *failure;
 
 	g_return_val_if_fail (drive != NULL, FALSE);
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->udi)
+	if (!priv->gdrive)
 		return FALSE;
 
-	watch = brasero_hal_watch_get_default ();
-	ctx = brasero_hal_watch_get_ctx (watch);
+	device = brasero_drive_get_device (drive);
+	handle = brasero_device_handle_open (device, FALSE, NULL);
+	if (!handle)
+		return FALSE;
 
-	dbus_error_init (&error);
-	result = libhal_device_lock (ctx,
-				     priv->udi,
-				     reason,
-				     &failure,
-				     &error);
-
-	if (dbus_error_is_set (&error))
-		dbus_error_free (&error);
-
-	if (reason_for_failure)
-		*reason_for_failure = g_strdup (failure);
-
-	if (failure)
-		dbus_free (failure);
-
-	if (result) {
-		BRASERO_MEDIA_LOG ("Device locked");
-	}
-	else {
+	result = (brasero_sbc_medium_removal (handle, 1, NULL) == BRASERO_SCSI_OK);
+	if (!result) {
 		BRASERO_MEDIA_LOG ("Device failed to lock");
 	}
+	else
+		BRASERO_MEDIA_LOG ("Device locked");
 
+	brasero_device_handle_close (handle);
 	return result;
 }
 
@@ -450,31 +393,31 @@ brasero_drive_lock (BraseroDrive *drive,
 gboolean
 brasero_drive_unlock (BraseroDrive *drive)
 {
+	BraseroDeviceHandle *handle;
 	BraseroDrivePrivate *priv;
-	BraseroHALWatch *watch;
-	LibHalContext *ctx;
-	DBusError error;
+	const gchar *device;
 	gboolean result;
 
 	g_return_val_if_fail (drive != NULL, FALSE);
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), FALSE);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->udi)
+	if (!priv->gdrive)
 		return FALSE;
 
-	watch = brasero_hal_watch_get_default ();
-	ctx = brasero_hal_watch_get_ctx (watch);
+	device = brasero_drive_get_device (drive);
+	handle = brasero_device_handle_open (device, FALSE, NULL);
+	if (!handle)
+		return FALSE;
 
-	dbus_error_init (&error);
-	result = libhal_device_unlock (ctx,
-				       priv->udi,
-				       &error);
+	result = (brasero_sbc_medium_removal (handle, 0, NULL) == BRASERO_SCSI_OK);
+	if (!result) {
+		BRASERO_MEDIA_LOG ("Device failed to unlock");
+	}
+	else
+		BRASERO_MEDIA_LOG ("Device unlocked");
 
-	if (dbus_error_is_set (&error))
-		dbus_error_free (&error);
-
-	BRASERO_MEDIA_LOG ("Device unlocked");
+	brasero_device_handle_close (handle);
 	return result;
 }
 
@@ -496,14 +439,14 @@ brasero_drive_get_display_name (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	if (!priv->udi) {
+	if (!priv->gdrive) {
 		/* Translators: This is a fake drive, a file, and means that
 		 * when we're writing, we're writing to a file and create an
 		 * image on the hard drive. */
 		return g_strdup (_("Image File"));
 	}
 
-	return g_strdup (priv->name);
+	return g_drive_get_name (priv->gdrive);
 }
 
 /**
@@ -523,7 +466,7 @@ brasero_drive_get_device (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
-	return priv->path;
+	return priv->path? priv->path:priv->block_path;
 }
 
 /**
@@ -556,6 +499,7 @@ brasero_drive_get_block_device (BraseroDrive *drive)
  * to uniquely identify the drive.
  *
  * Return value: a string holding the HAL udi. Not to be freed
+ * Deprecated since 2.27.3
  **/
 const gchar *
 brasero_drive_get_udi (BraseroDrive *drive)
@@ -568,6 +512,13 @@ brasero_drive_get_udi (BraseroDrive *drive)
 	g_return_val_if_fail (BRASERO_IS_DRIVE (drive), NULL);
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
+	if (!priv->gdrive)
+		return NULL;
+
+	if (priv->udi)
+		return priv->udi;
+
+	priv->udi = g_drive_get_identifier (priv->gdrive, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
 	return priv->udi;
 }
 
@@ -592,7 +543,7 @@ brasero_drive_get_medium (BraseroDrive *drive)
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
 
-	if (!priv->probed && priv->udi)
+	if (!priv->probed && priv->gdrive)
 		return NULL;
 
 	return priv->medium;
@@ -654,62 +605,6 @@ brasero_drive_init (BraseroDrive *object)
 	priv->cancel = g_cancellable_new ();
 }
 
-static void
-brasero_drive_finalize (GObject *object)
-{
-	BraseroDrivePrivate *priv;
-
-	priv = BRASERO_DRIVE_PRIVATE (object);
-
-	if (priv->name) {
-		libhal_free_string (priv->name);
-		priv->name = NULL;
-	}
-
-	if (priv->path) {
-		libhal_free_string (priv->path);
-		priv->path = NULL;
-	}
-
-	if (priv->block_path) {
-		libhal_free_string (priv->block_path);
-		priv->block_path = NULL;
-	}
-
-	if (priv->medium) {
-		g_object_unref (priv->medium);
-		priv->medium = NULL;
-	}
-
-	if (priv->hal_sig) {
-		BraseroHALWatch *watch;
-		LibHalContext *ctx;
-		DBusError error;
-
-		watch = brasero_hal_watch_get_default ();
-		ctx = brasero_hal_watch_get_ctx (watch);
-
-		dbus_error_init (&error);
-		libhal_device_remove_property_watch (ctx, priv->udi, &error);
-
-		g_signal_handler_disconnect (watch, priv->hal_sig);
-		priv->hal_sig = 0;
-	}
-
-	if (priv->udi) {
-		g_free (priv->udi);
-		priv->udi = NULL;
-	}
-
-	if (priv->cancel) {
-		g_cancellable_cancel (priv->cancel);
-		g_object_unref (priv->cancel);
-		priv->cancel = NULL;
-	}
-
-	G_OBJECT_CLASS (brasero_drive_parent_class)->finalize (object);
-}
-
 static void
 brasero_drive_medium_probed (BraseroMedium *medium,
 			     BraseroDrive *self)
@@ -789,60 +684,19 @@ brasero_drive_reprobe (BraseroDrive *drive)
 			  drive);
 }
 
-#if 0
-
-void
-brasero_drive_hal_reprobe (BraseroDrive *self)
-{
-	BraseroDrivePrivate *priv;
-	BraseroHALWatch *watch;
-	LibHalContext *ctx;
-	DBusError error;
-
-	priv = BRASERO_DRIVE_PRIVATE (self);
-
-	watch = brasero_hal_watch_get_default ();
-	ctx = brasero_hal_watch_get_ctx (watch);
-
-	dbus_error_init (&error);
-	if (!libhal_device_reprobe (ctx, priv->udi, &error)) {
-		BRASERO_MEDIA_LOG ("libhal_device_reprobe () failed %s",
-				  error.message);
-		dbus_error_free (&error);
-	}			      
-}
-
-#endif
-
 static void
-brasero_drive_check_medium_inside (BraseroDrive *self)
+brasero_drive_check_medium_inside_gdrive (BraseroDrive *self)
 {
 	BraseroDrivePrivate *priv;
-	BraseroHALWatch *watch;
-	gboolean has_medium;
-	LibHalContext *ctx;
-	DBusError error;
 
 	priv = BRASERO_DRIVE_PRIVATE (self);
 
-	watch = brasero_hal_watch_get_default ();
-	ctx = brasero_hal_watch_get_ctx (watch);
+	BRASERO_MEDIA_LOG ("Contents changed %i", g_drive_has_media (priv->gdrive));
 
-	BRASERO_MEDIA_LOG ("Contents changed");
+	if (g_drive_has_media (priv->gdrive)) {
+		if (priv->medium)
+			return;
 
-	dbus_error_init (&error);
-	has_medium = libhal_device_get_property_bool (ctx,
-						      priv->udi,
-						      "storage.removable.media_available",
-						      &error);
-	if (dbus_error_is_set (&error)) {
-		g_warning ("Hal connection problem :  %s\n",
-			   error.message);
-		dbus_error_free (&error);
-		return;
-	}
-
-	if (has_medium) {
 		BRASERO_MEDIA_LOG ("Medium inserted");
 
 		priv->probed = FALSE;
@@ -875,127 +729,235 @@ brasero_drive_check_medium_inside (BraseroDrive *self)
 }
 
 static void
-brasero_drive_medium_inside_property_changed_cb (BraseroHALWatch *watch,
-						 const char *udi,
-						 const char *key,
-						 BraseroDrive *drive)
+brasero_drive_medium_gdrive_changed_cb (BraseroDrive *gdrive,
+					BraseroDrive *drive)
 {
+	brasero_drive_check_medium_inside_gdrive (drive);
+}
+
+static gboolean
+brasero_drive_probed (gpointer data)
+{
+	BraseroDrive *drive = BRASERO_DRIVE (data);
 	BraseroDrivePrivate *priv;
 
+	priv = BRASERO_DRIVE_PRIVATE (data);
+
+	g_thread_join (priv->probe);
+	priv->probe = NULL;
+
+	/* If it's not a fake drive then connect to signal for any
+	 * change and check medium inside */
+	g_signal_connect (priv->gdrive,
+			  "changed",
+			  G_CALLBACK (brasero_drive_medium_gdrive_changed_cb),
+			  drive);
+
+	brasero_drive_check_medium_inside_gdrive (drive);
+
+	priv->probe_id = 0;
+	return FALSE;
+}
+
+static gboolean
+brasero_drive_get_caps_profiles (BraseroDrive *self,
+                                 BraseroDeviceHandle *handle,
+                                 BraseroScsiErrCode *code)
+{
+	BraseroScsiGetConfigHdr *hdr = NULL;
+	BraseroScsiProfileDesc *profiles;
+	BraseroScsiFeatureDesc *desc;
+	BraseroDrivePrivate *priv;
+	BraseroScsiResult result;
+	int profiles_num;
+	int size;
+
+	priv = BRASERO_DRIVE_PRIVATE (self);
+
+	BRASERO_MEDIA_LOG ("Checking supported profiles");
+	result = brasero_mmc2_get_configuration_feature (handle,
+	                                                 BRASERO_SCSI_FEAT_PROFILES,
+	                                                 &hdr,
+	                                                 &size,
+	                                                 code);
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_MEDIA_LOG ("GET CONFIGURATION failed");
+		return FALSE;
+	}
+
+	BRASERO_MEDIA_LOG ("Dectected media %x", BRASERO_GET_16 (hdr->current_profile));
+
+	/* Go through all features available */
+	desc = hdr->desc;
+	profiles = (BraseroScsiProfileDesc *) desc->data;
+	profiles_num = desc->add_len / sizeof (BraseroScsiProfileDesc);
+
+	while (profiles_num) {
+		switch (BRASERO_GET_16 (profiles->number)) {
+			case BRASERO_SCSI_PROF_CDR:
+				priv->caps |= BRASERO_DRIVE_CAPS_CDR;
+				break;
+			case BRASERO_SCSI_PROF_CDRW:
+				priv->caps |= BRASERO_DRIVE_CAPS_CDRW;
+				break;
+			case BRASERO_SCSI_PROF_DVD_R: 
+				priv->caps |= BRASERO_DRIVE_CAPS_DVDR;
+				break;
+			case BRASERO_SCSI_PROF_DVD_RW_SEQUENTIAL: 
+			case BRASERO_SCSI_PROF_DVD_RW_RESTRICTED: 
+				priv->caps |= BRASERO_DRIVE_CAPS_DVDRW;
+				break;
+			case BRASERO_SCSI_PROF_DVD_RAM: 
+				priv->caps |= BRASERO_DRIVE_CAPS_DVDRAM;
+				break;
+			case BRASERO_SCSI_PROF_DVD_R_PLUS_DL:
+				priv->caps |= BRASERO_DRIVE_CAPS_DVDR_PLUS_DL;
+				break;
+			case BRASERO_SCSI_PROF_DVD_RW_PLUS_DL:
+				priv->caps |= BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL;
+				break;
+			case BRASERO_SCSI_PROF_DVD_R_PLUS:
+				priv->caps |= BRASERO_DRIVE_CAPS_DVDR_PLUS;
+				break;
+			case BRASERO_SCSI_PROF_DVD_RW_PLUS:
+				priv->caps |= BRASERO_DRIVE_CAPS_DVDRW_PLUS;
+				break;
+			case BRASERO_SCSI_PROF_BR_R_SEQUENTIAL:
+			case BRASERO_SCSI_PROF_BR_R_RANDOM:
+				priv->caps |= BRASERO_DRIVE_CAPS_BDR;
+				break;
+			case BRASERO_SCSI_PROF_BD_RW:
+				priv->caps |= BRASERO_DRIVE_CAPS_BDRW;
+				break;
+			default:
+				break;
+		}
+
+		if (priv->probe_cancelled)
+			break;
+
+		/* Move the pointer to the next features */
+		profiles ++;
+		profiles_num --;
+	}
+
+	g_free (hdr);
+	return TRUE;
+}
+
+static void
+brasero_drive_get_caps_2A (BraseroDrive *self,
+                           BraseroDeviceHandle *handle,
+                           BraseroScsiErrCode *code)
+{
+	BraseroScsiStatusPage *page_2A = NULL;
+	BraseroScsiModeData *data = NULL;
+	BraseroDrivePrivate *priv;
+	BraseroScsiResult result;
+	int size = 0;
+
+	priv = BRASERO_DRIVE_PRIVATE (self);
+
+	result = brasero_spc1_mode_sense_get_page (handle,
+						   BRASERO_SPC_PAGE_STATUS,
+						   &data,
+						   &size,
+						   code);
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_MEDIA_LOG ("MODE SENSE failed");
+		return;
+	}
+
+	page_2A = (BraseroScsiStatusPage *) &data->page;
+
+	if (page_2A->wr_CDR != 0)
+		priv->caps |= BRASERO_DRIVE_CAPS_CDR;
+	if (page_2A->wr_CDRW != 0)
+		priv->caps |= BRASERO_DRIVE_CAPS_CDRW;
+	if (page_2A->wr_DVDR != 0)
+		priv->caps |= BRASERO_DRIVE_CAPS_DVDR;
+	if (page_2A->wr_DVDRAM != 0)
+		priv->caps |= BRASERO_DRIVE_CAPS_DVDRAM;
+
+	g_free (data);
+}
+
+static gpointer
+brasero_drive_probe_thread (gpointer data)
+{
+	gint counter = 0;
+	const gchar *path;
+	BraseroScsiErrCode code;
+	BraseroDrivePrivate *priv;
+	BraseroDeviceHandle *handle;
+	BraseroDrive *drive = BRASERO_DRIVE (data);
+
 	priv = BRASERO_DRIVE_PRIVATE (drive);
+	path = brasero_drive_get_device (drive);
+	if (!path)
+		path = brasero_drive_get_block_device (drive);
 
-	if (key && strcmp (key, "storage.removable.media_available")) {
-		priv->probed = TRUE;
-		return;
+	/* the drive might be busy (a burning is going on) so we don't block
+	 * but we re-try to open it every second */
+	BRASERO_MEDIA_LOG ("Trying to open device %s", path);
+
+	handle = brasero_device_handle_open (path, FALSE, &code);
+	while (!handle && counter <= BRASERO_DRIVE_OPEN_ATTEMPTS) {
+		sleep (1);
+
+		if (priv->probe_cancelled) {
+			BRASERO_MEDIA_LOG ("Open () cancelled");
+			priv->probe = NULL;
+			return NULL;
+		}
+
+		counter ++;
+		handle = brasero_device_handle_open (path, FALSE, &code);
 	}
 
-	if (udi && strcmp (udi, priv->udi)) {
-		priv->probed = TRUE;
-		return;
+	if (priv->probe_cancelled) {
+		BRASERO_MEDIA_LOG ("Open () cancelled");
+		priv->probe = NULL;
+		return NULL;
 	}
 
-	brasero_drive_check_medium_inside (drive);
+	if (handle) {
+		BRASERO_MEDIA_LOG ("Open () succeeded");
+
+		if (!brasero_drive_get_caps_profiles (drive, handle, &code))
+			brasero_drive_get_caps_2A (drive, handle, &code);
+
+		brasero_device_handle_close (handle);
+
+		BRASERO_MEDIA_LOG ("Drive caps are %d", priv->caps);
+	}
+	else
+		BRASERO_MEDIA_LOG ("Open () failed: medium busy");
+
+	priv->probe_id = g_idle_add (brasero_drive_probed, drive);
+	return NULL;
 }
 
 static void
 brasero_drive_init_real (BraseroDrive *drive)
 {
 	BraseroDrivePrivate *priv;
-	BraseroHALWatch *watch;
-	LibHalContext *ctx;
-	DBusError error;
-	char *parent;
 
 	priv = BRASERO_DRIVE_PRIVATE (drive);
 
-	watch = brasero_hal_watch_get_default ();
-	ctx = brasero_hal_watch_get_ctx (watch);
+	priv->block_path = g_drive_get_identifier (priv->gdrive, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 
-	priv->name = libhal_device_get_property_string (ctx,
-							priv->udi,
-							DEVICE_MODEL,
-							NULL);
+	BRASERO_MEDIA_LOG ("Initializing drive %s", priv->block_path);
 
-	priv->path = libhal_device_get_property_string (ctx,
-							priv->udi,
-							BLOCK_DEVICE,
-							NULL);
-	if (priv->path && priv->path [0] == '\0') {
-		g_free (priv->path);
-		priv->path = NULL;
-	}
-
-	priv->block_path = libhal_device_get_property_string (ctx,
-							      priv->udi,
-							      "block.device",
-							      NULL);
-	if (priv->block_path && priv->block_path [0] == '\0') {
-		g_free (priv->block_path);
-		priv->block_path = NULL;
-	}
-
-	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.cdr", NULL))
-		priv->caps |= BRASERO_DRIVE_CAPS_CDR;
-	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.cdrw", NULL))
-		priv->caps |= BRASERO_DRIVE_CAPS_CDRW;
-	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.dvdr", NULL))
-		priv->caps |= BRASERO_DRIVE_CAPS_DVDR;
-	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.dvdrw", NULL))
-		priv->caps |= BRASERO_DRIVE_CAPS_DVDRW;
-	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.dvdplusr", NULL))
-		priv->caps |= BRASERO_DRIVE_CAPS_DVDR_PLUS;
-	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.dvdplusrw", NULL))
-		priv->caps |= BRASERO_DRIVE_CAPS_DVDRW_PLUS;
-	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.dvdplusrdl", NULL))
-		priv->caps |= BRASERO_DRIVE_CAPS_DVDR_PLUS_DL;
-	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.dvdplusrwdl", NULL))
-		priv->caps |= BRASERO_DRIVE_CAPS_DVDRW_PLUS_DL;
-	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.dvdram", NULL))
-		priv->caps |= BRASERO_DRIVE_CAPS_DVDRAM;
-	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.bdr", NULL))
-		priv->caps |= BRASERO_DRIVE_CAPS_BDR;
-	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.bdre", NULL))
-		priv->caps |= BRASERO_DRIVE_CAPS_BDRW;
-
-	BRASERO_MEDIA_LOG ("Drive caps are %d", priv->caps);
-
-	/* Also get its parent to retrieve the bus, host, lun values */
-	priv->bus = -1;
-	priv->lun = -1;
-	priv->target = -1;
-	parent = libhal_device_get_property_string (ctx, priv->udi, "info.parent", NULL);
-	if (parent) {
-		/* Check it is a SCSI interface */
-		if (libhal_device_property_exists (ctx, parent, "scsi.host", NULL)
-		&&  libhal_device_property_exists (ctx, parent, "scsi.lun", NULL)
-		&&  libhal_device_property_exists (ctx, parent, "scsi.target", NULL)) {
-			priv->bus = libhal_device_get_property_int (ctx, parent, "scsi.host", NULL);
-			priv->lun = libhal_device_get_property_int (ctx, parent, "scsi.lun", NULL);
-			priv->target = libhal_device_get_property_int (ctx, parent, "scsi.target", NULL);
-		}
-
-		BRASERO_MEDIA_LOG ("Drive %s has bus,target,lun = %i %i %i",
-				  priv->path,
-				  priv->bus,
-				  priv->target,
-				  priv->lun);
-		libhal_free_string (parent);
-	}
-
-	/* Now check for the medium */
-	brasero_drive_check_medium_inside (drive);
-
-	dbus_error_init (&error);
-	libhal_device_add_property_watch (ctx, priv->udi, &error);
-	if (dbus_error_is_set (&error)) {
-		g_warning ("Failed to watch property : %s\n", error.message);
-		dbus_error_free (&error);
-	}
-
-	priv->hal_sig = g_signal_connect (watch,
-					  "property-changed",
-					  G_CALLBACK (brasero_drive_medium_inside_property_changed_cb),
-					  drive);
+	/* NOTE: why a thread? Because in case of a damaged medium, brasero can
+	 * block on some functions until timeout and if we do this in the main
+	 * thread then our whole UI blocks. This medium won't be exported by the
+	 * BraseroDrive that exported until it returns PROBED signal.
+	 * One (good) side effect is that it also improves start time. */
+	priv->probe = g_thread_create (brasero_drive_probe_thread,
+				       drive,
+				       TRUE,
+				       NULL);
 }
 
 static void
@@ -1013,8 +975,10 @@ brasero_drive_set_property (GObject *object,
 	switch (prop_id)
 	{
 	case PROP_UDI:
-		priv->udi = g_strdup (g_value_get_string (value));
-		if (!priv->udi) {
+		break;
+	case PROP_GDRIVE:
+		priv->gdrive = g_value_get_object (value);
+		if (!priv->gdrive) {
 			priv->probed = TRUE;
 			priv->medium = g_object_new (BRASERO_TYPE_VOLUME,
 						     "drive", object,
@@ -1022,7 +986,7 @@ brasero_drive_set_property (GObject *object,
 		}
 		else
 			brasero_drive_init_real (BRASERO_DRIVE (object));
-			
+
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1045,7 +1009,9 @@ brasero_drive_get_property (GObject *object,
 	switch (prop_id)
 	{
 	case PROP_UDI:
-		g_value_set_string (value, g_strdup (priv->udi));
+		break;
+	case PROP_GDRIVE:
+		g_value_set_object (value, priv->gdrive);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -1053,6 +1019,56 @@ brasero_drive_get_property (GObject *object,
 	}
 }
 
+static void
+brasero_drive_finalize (GObject *object)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (object);
+
+	if (priv->udi) {
+		g_free (priv->udi);
+		priv->udi = NULL;
+	}
+
+	if (priv->probe) {
+		priv->probe_cancelled = TRUE;
+		g_thread_join (priv->probe);
+		priv->probe = 0;
+	}
+
+	if (priv->probe_id) {
+		g_source_remove (priv->probe_id);
+		priv->probe_id = 0;
+	}
+
+	if (priv->block_path) {
+		g_free (priv->block_path);
+		priv->block_path = NULL;
+	}
+
+	if (priv->medium) {
+		g_object_unref (priv->medium);
+		priv->medium = NULL;
+	}
+
+	if (priv->gdrive) {
+		g_signal_handlers_disconnect_by_func (priv->gdrive,
+						      brasero_drive_medium_gdrive_changed_cb,
+						      object);
+		g_object_unref (priv->gdrive);
+		priv->gdrive = NULL;
+	}
+
+	if (priv->cancel) {
+		g_cancellable_cancel (priv->cancel);
+		g_object_unref (priv->cancel);
+		priv->cancel = NULL;
+	}
+
+	G_OBJECT_CLASS (brasero_drive_parent_class)->finalize (object);
+}
+
 static void
 brasero_drive_class_init (BraseroDriveClass *klass)
 {
@@ -1103,20 +1119,16 @@ brasero_drive_class_init (BraseroDriveClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_UDI,
 	                                 g_param_spec_string("udi",
-	                                                     "HAL udi",
-	                                                     "HAL udi as a string",
+	                                                     "Deprecated",
+	                                                     "HAL udi as a string (Deprecated)",
 	                                                     NULL,
 	                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class,
+	                                 PROP_GDRIVE,
+	                                 g_param_spec_object ("gdrive",
+	                                                      "GDrive",
+	                                                      "A GDrive object for the drive",
+	                                                      G_TYPE_DRIVE,
+	                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
-/**
- * This is not public API. Declared in burn-monitor.h.
- */
-
-BraseroDrive *
-brasero_drive_new (const gchar *udi)
-{
-	return g_object_new (BRASERO_TYPE_DRIVE,
-			     "udi", udi,
-			     NULL);
-}
