@@ -1,24 +1,28 @@
-/***************************************************************************
- *            burn-libisofs.c
- *
- *  lun aoû 21 14:34:32 2006
- *  Copyright  2006  Rouquier Philippe
- *  bonfire-app@wanadoo.fr
- ***************************************************************************/
-
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
- *  Brasero is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * Libbrasero-burn
+ * Copyright (C) Philippe Rouquier 2005-2009 <bonfire-app@wanadoo.fr>
  *
- *  Brasero is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * Libbrasero-burn is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to:
+ * The Libbrasero-burn authors hereby grant permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Libbrasero-burn. This permission is above and beyond the permissions granted
+ * by the GPL license by which Libbrasero-burn is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ * 
+ * Libbrasero-burn is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to:
  * 	The Free Software Foundation, Inc.,
  * 	51 Franklin Street, Fifth Floor
  * 	Boston, MA  02110-1301, USA.
@@ -674,24 +678,19 @@ brasero_libisofs_create_volume_thread (gpointer data)
 			 * children of the parent directory. If there is a
 			 * sibling destroy it. */
 			sibling = NULL;
-			iso_dir_find_children (ISO_DIR (parent),
-					       iso_new_find_conditions_name (path_name),
-					       &sibling);
-			if (sibling) {
-				IsoNode *node;
+			iso_dir_get_children (ISO_DIR (parent), &sibling);
 
-				BRASERO_JOB_LOG (self,
-						 "Looking for sibling for %s",
-						 path_name);
+			IsoNode *node;
+			while (iso_dir_iter_next (sibling, &node) == 1) {
+				const gchar *iso_name;
 
-				while (iso_dir_iter_next (sibling, &node) == 1) {
+				/* check if it has the same name */
+				iso_name = iso_node_get_name (node);
+				if (iso_name && !strcmp (iso_name, path_name))
 					BRASERO_JOB_LOG (self,
 							 "Found sibling for %s: removing %x",
 							 path_name,
 							 iso_dir_iter_remove (sibling));
-				}
-
-				iso_dir_iter_free (sibling);
 			}
 
 			if  (is_directory) {
@@ -733,8 +732,9 @@ brasero_libisofs_create_volume_thread (gpointer data)
 				IsoNode *node;
 				int err;
 
-				err = iso_tree_add_node (image,
+				err = iso_tree_add_new_node (image,
 							 ISO_DIR (parent),
+				                         path_name,
 							 local_path,
 							 &node);
 				if (err < 0) {
