@@ -72,29 +72,15 @@ struct _BraseroDiscIface {
 
 	/* signals */
 	void	(*selection_changed)			(BraseroDisc *disc);
-	void	(*contents_changed)			(BraseroDisc *disc,
-							 gint nb_files);
-	void	(*size_changed)				(BraseroDisc *disc,
-							 gint64 size);
-	void	(*flags_changed)			(BraseroDisc *disc,
-							 BraseroBurnFlag flags);
 
 	/* virtual functions */
-	BraseroDiscResult	(*get_status)		(BraseroDisc *disc,
-							 gint *remaining,
-							 gchar **current_task);
-
-	BraseroDiscResult	(*load_track)		(BraseroDisc *disc,
-							 BraseroDiscTrack *track);
-	BraseroDiscResult	(*get_track)		(BraseroDisc *disc,
-							 BraseroDiscTrack *track);
-
 	BraseroDiscResult	(*set_session_contents)	(BraseroDisc *disc,
 							 BraseroBurnSession *session);
 
 	BraseroDiscResult	(*add_uri)		(BraseroDisc *disc,
 							 const gchar *uri);
 
+	gboolean		(*is_empty)		(BraseroDisc *disc);
 	gboolean		(*get_selected_uri)	(BraseroDisc *disc,
 							 gchar **uri);
 	gboolean		(*get_boundaries)	(BraseroDisc *disc,
@@ -103,7 +89,6 @@ struct _BraseroDiscIface {
 
 	void			(*delete_selected)	(BraseroDisc *disc);
 	void			(*clear)		(BraseroDisc *disc);
-	void			(*reset)		(BraseroDisc *disc);
 
 	guint			(*add_ui)		(BraseroDisc *disc,
 							 GtkUIManager *manager,
@@ -130,41 +115,23 @@ brasero_disc_get_boundaries (BraseroDisc *disc,
 
 void
 brasero_disc_delete_selected (BraseroDisc *disc);
-void
+
+gboolean
 brasero_disc_clear (BraseroDisc *disc);
-void
-brasero_disc_reset (BraseroDisc *disc);
 
 BraseroDiscResult
 brasero_disc_get_status (BraseroDisc *disc,
 			 gint *remaining,
 			 gchar **current_task);
 
-BraseroDiscResult
-brasero_disc_get_track (BraseroDisc *disc,
-			BraseroDiscTrack *track);
-BraseroDiscResult
-brasero_disc_load_track (BraseroDisc *disc,
-			 BraseroDiscTrack *track);
-
 BraseroDiscResult
 brasero_disc_set_session_contents (BraseroDisc *disc,
 				   BraseroBurnSession *session);
 
-void
-brasero_disc_size_changed (BraseroDisc *disc,
-			   gint64 size);
-void
-brasero_disc_flags_changed (BraseroDisc *disc,
-			    BraseroBurnFlag flags);
-void
-brasero_disc_contents_changed (BraseroDisc *disc,
-			       gint nb_files);
+gboolean
+brasero_disc_is_empty (BraseroDisc *disc);
+
 void
 brasero_disc_selection_changed (BraseroDisc *disc);
 
-
-GtkWidget *
-brasero_disc_get_use_info_notebook (void);
-
 #endif /* DISC_H */
