@@ -35,11 +35,12 @@ fy
 
 #include <gconf/gconf-client.h>
 
+#include "brasero-misc.h"
+
 #include "brasero-plugin-manager-ui.h"
-#include "brasero-utils.h"
-#include "burn-plugin.h"
-#include "burn-caps.h"
-#include "burn-plugin-private.h"
+
+#include "brasero-plugin.h"
+#include "brasero-plugin-information.h"
 #include "burn-plugin-manager.h"
 #include "brasero-plugin-option.h"
 
@@ -345,7 +346,6 @@ plugin_manager_ui_populate_lists (BraseroPluginManagerUI *pm)
 {
 	BraseroPluginManagerUIPrivate *priv;
 	BraseroPlugin *plugin;
-	BraseroBurnCaps *caps;
 	const GSList *plugins;
 	GtkListStore *model;
 	GtkTreeIter iter;
@@ -353,8 +353,6 @@ plugin_manager_ui_populate_lists (BraseroPluginManagerUI *pm)
 	priv = BRASERO_PLUGIN_MANAGER_UI_GET_PRIVATE (pm);
 	plugins = priv->plugins;
 
-	caps = brasero_burn_caps_get_default ();
-
 	model = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (priv->tree)));
 
 	for (; plugins; plugins = plugins->next) {
@@ -371,7 +369,6 @@ plugin_manager_ui_populate_lists (BraseroPluginManagerUI *pm)
 				    PLUGIN_COLUMN, plugin,
 				    -1);
 	}
-	g_object_unref (caps);
 
 	plugin = NULL;
 	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter))
